/**
 * The MIT License (MIT)
 *
 * Copyright (c) 2014-2017 Marc de Verdelhan, 2017-2021 Ta4j Organization & respective
 * authors (see AUTHORS)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package org.ta4j.core.analysis.criteria;

import org.ta4j.core.BarSeries;
import org.ta4j.core.Position;
import org.ta4j.core.Trade;
import org.ta4j.core.TradingRecord;
import org.ta4j.core.num.Num;

/**
 * Buy and hold criterion.
 *
 * Calculates the return if a buy-and-hold strategy was used, buying on the
 * first bar and selling on the last bar.
 *
 * @see <a href=
 *      "http://en.wikipedia.org/wiki/Buy_and_hold">http://en.wikipedia.org/wiki/Buy_and_hold</a>
 */
public class BuyAndHoldReturnCriterion extends AbstractAnalysisCriterion {

    @Override
    public Num calculate(BarSeries series, Position position) {
        return createBuyAndHoldTrade(series, position.getEntry().getIndex(), position.getExit().getIndex())
                .getGrossReturn(series);
    }

    @Override
    public Num calculate(BarSeries series, TradingRecord tradingRecord) {
        return createBuyAndHoldTrade(series).getGrossReturn(series);
    }

    @Override
    public boolean betterThan(Num criterionValue1, Num criterionValue2) {
        return criterionValue1.isGreaterThan(criterionValue2);
    }

    private Position createBuyAndHoldTrade(BarSeries series) {
        return createBuyAndHoldTrade(series, series.getBeginIndex(), series.getEndIndex());
    }

    private Position createBuyAndHoldTrade(BarSeries series, int beginIndex, int endIndex) {
        Position position = new Position(Trade.TradeType.BUY);
        position.operate(beginIndex, series.getBar(beginIndex).getClosePrice(), series.numOf(1));
        position.operate(endIndex, series.getBar(endIndex).getClosePrice(), series.numOf(1));
        return position;
    }
}
